import json
import click
from flask import Blueprint
from .use_cases import NewRole, CreateUser
from .models import WhitelistToken

command_auth_mongo = Blueprint(
    'command_auth_mongo',
    __name__,
    cli_group='auth_mongo'
)


@command_auth_mongo.cli.command(
    'role-new',
    help="Create a Role"
)
@click.option('-n', '--name', 'name', required=True, prompt=True)
@click.option('-p', '--permissions', 'permissions',
              help='''JSON example: '{"key": "value"}' ''')
def role_new(**kwargs):
    request = dict()
    for (key, value) in kwargs.items():
        if value is not None:
            request[key] = value
    try:
        if 'permissions' in request:
            request['permissions'] = json.loads(request['permissions'])

        use_case = NewRole()
        result = use_case.handle(request)

        click.echo("Result of Role creation: {}. {}".format(
            result.message,
            result.errors if result.errors else ''
        ))
    except Exception as e:
        click.echo("Create a Role ERROR: {}".format(e))


@command_auth_mongo.cli.command(
    'user-new',
    help='Create User'
)
@click.option('-r', '--role', required=True, prompt='Role',
              help='User role')
@click.option('-u', '--username', required=True, prompt='Username',
              help='Your username to login')
@click.option('--password', required=True, prompt=True,
              hide_input=True, confirmation_prompt=True,
              help='Your password to login')
def user_new(
    role: str,
    username: str,
    password: str
):
    click.echo('Creation of user started.')
    use_case = CreateUser()
    request = dict(
        role=role,
        username=username,
        password=password,
        password_confirmed=password
    )
    result = use_case.handle(request)

    click.echo("Result of User creation: {}. {}".format(
        result.message,
        result.errors if result.errors else ''
    ))


@command_auth_mongo.cli.command(
    'clear-tokens',
    help='Clear all tokens of the WhitelistToken'
)
@click.option('-f', '--forced', is_flag=True)
def clear_tokens(forced):
    if forced:
        WhitelistToken.drop_collection()
    else:
        try:
            _ = WhitelistToken.objects().first()
        except Exception:
            # There are changes in the model
            # Delete collection to avoid problems
            WhitelistToken.drop_collection()

    click.echo('Token cleanup completed')



from unittest import TestCase
from mongoengine import connect, disconnect
from flask_auth_service_mongo.security.models import User, Role


class TestRole(TestCase):
    @classmethod
    def setUpClass(cls):
        disconnect()
        connect('mongoenginetest', host='mongomock://localhost')

    @classmethod
    def tearDownClass(cls):
        disconnect()

    def test_save(self):
        params = dict(
            name='admin',
            permissions=dict(
                x=True
            )
        )

        role = Role(**params)
        role.save()

        result = Role.objects().first()

        self.assertIsInstance(result, Role)
        for key in params:
            self.assertEqual(getattr(result, key), params[key])


class TestUser(TestCase):
    @classmethod
    def setUpClass(cls):
        disconnect()
        connect('mongoenginetest', host='mongomock://localhost')

    @classmethod
    def tearDownClass(cls):
        disconnect()

    def test_save(self):
        # Prepara un role
        role = Role(name='admin', permissions={'a': 1})
        role.save()

        params = dict(
            username='a name',
            password='a pass'
        )
        user = User(**params)
        self.assertIsNone(user.role)

        user.role = role
        user.save()

        result = User.objects().first()

        self.assertIsInstance(result, User)
        self.assertIsInstance(result.role, Role)



import json
import graphene
from unittest import TestCase
from unittest.mock import patch, Mock
from flask_auth_service_mongo.security.schema import (
    UpdatePassword,
    UpdateUser,
    ResetPassword,
)
from flask_auth_service_mongo.security.models import User


def resolve_patch(patch: str) -> str:
    return 'flask_auth_service_mongo.security.schema.user.{}'.format(patch)


class Mutation(graphene.ObjectType):
    updatePassword = UpdatePassword.Field()
    updateUser = UpdateUser.Field()
    resetPassword = ResetPassword.Field()


class TestMutationUpdatePassword(TestCase):

    __query = '''
        mutation {
            updatePassword (input: {
                currentPassword: "currentPass"
                newPassword: "newPass"
                passwordConfirmed: "newPass"
            }) {
                ok
            }
        }
    '''

    @patch(resolve_patch('EditPasswordUser'))
    @patch(resolve_patch('auth.current_user'))
    def test_error(self, mock_auth, mock_use_case):
        # Mocks
        mock_auth.return_value = Mock(id='abc123')
        mock_handle = Mock()
        mock_handle.handle.return_value = Mock(
            http_code=400,
            message='error'
        )
        mock_use_case.return_value = mock_handle

        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)

        # Asserts
        self.assertIsNotNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"updatePassword": null}'''
        )
        mock_handle.handle.assert_called_with({
            'current_password': 'currentPass',
            'new_password': 'newPass',
            'password_confirmed': 'newPass',
            'id': 'abc123'
        })

    @patch(resolve_patch('EditPasswordUser'))
    @patch(resolve_patch('auth.current_user'))
    def test_ok(self, mock_auth, mock_use_case):
        # Mocks
        mock_auth.return_value = Mock(id='abc123')
        mock_handle = Mock()
        mock_handle.handle.return_value = Mock(
            http_code=200
        )
        mock_use_case.return_value = mock_handle

        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)

        # Asserts
        self.assertIsNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"updatePassword": {"ok": true}}'''
        )


class TestMutationUpdateUser(TestCase):

    __query = '''
        mutation {
            updateUser (input: {
                id: "VXNlcjo1ZTY2NmZmYTYyMjUzZTgyZjVjYWVhNDE="
                changePassword: false
            }) {
                user {
                    username
                }
            }
        }
    '''

    @patch(resolve_patch('EditUserUseCase'))
    def test_error(self, mock_use_case):
        # Mocks
        mock_handle = Mock()
        mock_handle.handle.return_value = Mock(
            http_code=400,
            message='error'
        )
        mock_use_case.return_value = mock_handle

        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)

        # Asserts
        self.assertIsNotNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"updateUser": null}'''
        )
        mock_handle.handle.assert_called_with({
            'id': '5e666ffa62253e82f5caea41',
            'change_password': False
        })

    @patch(resolve_patch('EditUserUseCase'))
    def test_ok(self, mock_use_case):
        # Mocks
        mock_handle = Mock()
        mock_handle.handle.return_value = Mock(
            http_code=200,
            user=User(
                username='user_name'
            )
        )
        mock_use_case.return_value = mock_handle

        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)

        # Asserts
        self.assertIsNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"updateUser": {"user": {"username": "user_name"}}}'''
        )
        mock_handle.handle.assert_called_with({
            'id': '5e666ffa62253e82f5caea41',
            'change_password': False
        })


class TestMutationResetPassword(TestCase):
    __query = '''
        mutation {
            resetPassword (input: {
                id: "VXNlcjo1ZTY2NmZmYTYyMjUzZTgyZjVjYWVhNDE="
            }) {
                password
            }
        }
    '''

    @patch(resolve_patch('ResetPasswordUseCase'))
    def test_ok(self, mock_case):
        # mocks
        mock_result = Mock()
        mock_response = Mock(
            message='ok',
            http_code=200,
            password='qwer'
        )
        mock_result.handle.return_value = mock_response
        mock_case.return_value = mock_result
        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)
        self.assertIsNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"resetPassword": {"password": "qwer"}}'''
        )
        mock_result.handle.assert_called_with(
            {'id': '5e666ffa62253e82f5caea41'})

    @patch(resolve_patch('ResetPasswordUseCase'))
    def test_error(self, mock_case):
        # mocks
        mock_result = Mock()
        mock_response = Mock(
            message='not_found',
            http_code=404,
            password=None
        )
        mock_result.handle.return_value = mock_response
        mock_case.return_value = mock_result
        # Execution
        schema = graphene.Schema(mutation=Mutation)
        result = schema.execute(self.__query)

        self.assertIsNotNone(result.errors)
        self.assertEqual(
            json.dumps(result.data),
            '''{"resetPassword": null}'''
        )
        mock_result.handle.assert_called_with(
            {'id': '5e666ffa62253e82f5caea41'})



from flask import request
from ...config import WHITE_LIST_TOKEN, REQUIRE_PASSWORD_CHANGE
from ...constants import responses
from ...constants.enums import HttpCode
from ..utils import token_decode
from ..repository import UserRepository, WhitelistTokenRepository
from .user import set_current_user


def _response_unauthorized(message: str = None) -> (dict, int):
    """Crea una respuesta de "unauthorized"

    Args:
        message (str): Default None

    Returns:
        body (dict)
        http_code (int)
    """
    return dict(
        message=message if message else responses.UNAUTHORIZED
    ), HttpCode.UNAUTHORIZED.value


def required(role: str = None, require_password_change: bool = None):
    """Decorador para validar token jwt.
    Si role es diferente de None, verifica tambien el role

    Args:
        role (str): Or List[str]
        require_password_change (bool):
            Default = config.REQUIRE_PASSWORD_CHANGE.
            Si esta activo y User.change_password es True rechaza la petición.

    Returns:
        (dict, int): (body, http_code)
    """
    if isinstance(role, str):
        role = [role]

    if require_password_change is None:
        require_password_change = REQUIRE_PASSWORD_CHANGE

    def decorator(func):
        def wrapper(*args, **kwargs):
            authorization = request.headers.get('Authorization')
            if not authorization:
                # header sin "Authorization"
                return _response_unauthorized()

            auth_type, token = authorization.split(' ')
            if auth_type != 'Bearer':
                # Tipo de "Authorization" no valido
                return _response_unauthorized()

            payload = token_decode(token)
            if payload.error:
                # Error de token
                return _response_unauthorized(payload.error)

            if WHITE_LIST_TOKEN:
                in_whitelist = WhitelistTokenRepository.find_one(
                    uuid_access=payload.uuid
                )
                if not in_whitelist:
                    # El token no esta en la lista blanca
                    return _response_unauthorized()

            user = UserRepository.find(payload.user_id)
            if not user:
                # Usuario no encontrado
                return _response_unauthorized()

            if not user.active:
                # Usuario apagado
                return _response_unauthorized()

            if role and user.role.name not in role:
                # Role incorrecto
                return _response_unauthorized()

            if require_password_change and user.change_password:
                # El usuario debe cambiar de contraseña
                return _response_unauthorized('password_change_required')

            set_current_user(user)

            return func(*args, **kwargs)
        wrapper.__doc__ = func.__doc__
        return wrapper
    return decorator



from functools import reduce
import re


def recursive_get(d: dict, keys, default=None):
    """Aplica un get recursivo al dict parametro

    Args:
        d (dict)
        keys (str|list|tuple): Si es un str keyA.keyB => [keyA, keyB]
        default (mixed): No requerido
    """
    if isinstance(keys, str):
        keys = keys.split('.')
    result = reduce(lambda c, k: c.get(k, {}), keys, d)
    if default is not None and result == {}:
        return default
    return result


def dict_keys_to_snake(data: dict) -> dict:
    """Convierte las keys de un diccionario de camel case a snake
    Args:
        data (dict)
    Returns:
        dict
    """
    return dict(
        zip([camel_to_snake(k) for k in data.keys()], list(data.values()))
    )


def camel_to_snake(msg: str):
    """Convierte un string camel case a snake
    Args:
        msg (str)
    Returns:
        str
    """
    msg = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', msg)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', msg).lower()



from mongoengine import (
    StringField,
    DictField,
    ReferenceField,
    DateTimeField,
    BooleanField,
    Document
)


class Role(Document):
    name = StringField(required=True, unique=True)
    permissions = DictField(required=False)


class User(Document):
    username = StringField(required=True, unique=True)
    password = StringField(required=True)
    active = BooleanField(default=True)

    change_password = BooleanField(default=False)
    """bool: Indica si es necesario realizar cambio de contraseña
    ::
        (default=True)
    """

    role = ReferenceField(
        Role,
        dbref=True,
        required=True
    )


class WhitelistToken(Document):
    uuid_access = StringField(required=True, unique=True)
    uuid_refresh = StringField(required=True, unique=True)
    user = ReferenceField(
        User,
        dbref=True,
        required=True
    )
    created_at = DateTimeField()



import string
import random
from ... import config
from ...constants import responses
from ...constants.enums import HttpCode
from ...utils.use_case import Response, UseCaseInterface
from ...utils.validator import Validator
from ..utils import password_hash, password_match
from ..models import User
from ..repository import UserRepository, RoleRepository


class ResetPasswordResponse(Response):
    """Response from use case resetPassword

    Attributes:
        password (str)
        http_code (int)
        message (str)
        errors (list)
    """
    def __init__(
        self,
        password: str = None,
        *args,
        **kwargs
    ):
        self.password = password

        super().__init__(*args, **kwargs)


class SaveUserResponse(Response):
    """Respuesta para los casos de uso new y update user

    Attributes:
        user (User)
        http_code (int)
        message (str)
        errors (list)
    """
    def __init__(self, user: User = None, *args, **kwargs):
        self.user = user

        super(SaveUserResponse, self).__init__(*args, **kwargs)


class CreateUser(UseCaseInterface):
    """Crea usuario"""

    def handle(self, request: dict) -> SaveUserResponse:
        """
        Args:
            request (dict): Ejemplo:
                ::
                    {
                        'role': 'role_name',
                        'username': 'username',
                        'password': 'pass',
                        'password_confirmed': 'pass',
                        'identifier': '123' (Optional)
                    }
        Returns:
            SaveUserResponse:
        """
        self._request = request

        valid, response = self._validate()
        if not valid:
            return response

        try:
            self._create_user()
            response = SaveUserResponse(
                message=responses.CREATED,
                http_code=HttpCode.CREATED,
                user=self._user
            )
        except Exception:
            response = SaveUserResponse(
                message=responses.INTERNAL_SERVER_ERROR,
                http_code=HttpCode.INTERNAL_SERVER_ERROR
            )

        return response

    def _validate(self) -> (bool, SaveUserResponse):
        """Validaciones

        Returns:
            (bool, SaveUserResponse): (True, None): Request valido.
            (False, SaveUserResponse) Request no valido.
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, SaveUserResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )
        self._username = self._request['username'].lower()

        if (self._request.get('password') !=
                self._request.get('password_confirmed')):
            return False, SaveUserResponse(
                message="Passwords don't match",
                http_code=HttpCode.NOT_ACCEPTABLE
            )

        self._role = RoleRepository.find_one(name=self._request['role'])
        if not self._role:
            return False, SaveUserResponse(
                message="Role does not exist",
                http_code=HttpCode.NOT_ACCEPTABLE
            )

        user = UserRepository.find_one(username=self._username)
        if user:
            return False, SaveUserResponse(
                message="Username already exists",
                http_code=HttpCode.NOT_ACCEPTABLE
            )

        return True, None

    def _create_user(self):
        """ Crea User """
        self._user = User(
            username=self._username,
            password=password_hash(self._request['password']),
            role=self._role
        )

        self._user.save()

    def __request_is_valid(self) -> (bool, dict):
        """ El request debe cumplir con la estructura,
        no debe tener keys adicionales

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            "role": {
                "type": "string",
                "required": True,
            },
            "username": {
                "type": "string",
                "minlength": int(config.USERNAME_MIN_LENGTH),
                "required": True,
                "no_spaces": True
            },
            "password": {
                "type": "string",
                "minlength": int(config.PASSWORD_MIN_LENGTH),
                "required": True,
                "no_spaces": True
            },
            "password_confirmed": {
                "type": "string",
                "required": True,
                "no_spaces": True
            }
        }

        valid = v.validate(self._request)
        return valid, v.errors


class EditPasswordUser(UseCaseInterface):
    """Editar contraseña del usuario"""

    def handle(self, request: dict) -> SaveUserResponse:
        """
        Args:
            request (dict): Ejemplo:
                ::
                    {
                        'id': '5e404653d1c098c5177cc69d',
                        'current_password': 'pass',
                        'new_password': 'new_pass',
                        'password_confirmed': 'new_pass'
                    }

        Returns:
            SaveUserResponse:
        """
        self._request = request
        self._user = None

        valid, response = self._validate()
        if not valid:
            return response

        try:
            self._edit()
            response = SaveUserResponse(
                message=responses.OK,
                http_code=HttpCode.OK,
                user=self._user
            )
        except Exception:
            response = SaveUserResponse(
                message=responses.INTERNAL_SERVER_ERROR,
                http_code=HttpCode.INTERNAL_SERVER_ERROR
            )

        return response

    def _edit(self) -> None:
        """Edita la contraseña del usuario con los datos del request"""
        self._user.password = password_hash(self._request['new_password'])
        self._user.change_password = False
        self._user.save()

    def _validate(self) -> (bool, SaveUserResponse):
        """Validaciones

        Returns:
            (bool, SaveUserResponse): (True, None) = Si es válido
                (False, obj) = No es válido
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, SaveUserResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )

        if (self._request.get('new_password') !=
                self._request.get('password_confirmed')):
            return False, SaveUserResponse(
                message="Passwords don't match",
                http_code=HttpCode.BAD_REQUEST
            )

        self._user = UserRepository.find(self._request['id'])
        if not self._user:
            return False, SaveUserResponse(
                message=responses.NOT_FOUND,
                http_code=HttpCode.NOT_FOUND
            )

        if not password_match(
            self._request['current_password'],
            self._user.password
        ):
            return False, SaveUserResponse(
                message="Current password don't match",
                http_code=HttpCode.BAD_REQUEST
            )

        if (self._request.get('current_password') ==
                self._request.get('new_password')):
            return False, SaveUserResponse(
                message="The new password is equal as the current password",
                http_code=HttpCode.BAD_REQUEST
            )

        return True, None

    def __request_is_valid(self) -> (bool, dict):
        """ El request debe cumplir con la estructura,
        no debe tener keys adicionales

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            "id": {
                'type': 'string',
                'required': True,
                'mongo_id': True
            },
            "current_password": {
                "type": "string",
                "required": True
            },
            "new_password": {
                "type": "string",
                "minlength": int(config.PASSWORD_MIN_LENGTH),
                "required": True,
                "no_spaces": True
            },
            "password_confirmed": {
                "type": "string",
                "required": True
            }
        }

        valid = v.validate(self._request)
        return valid, v.errors


class DeleteUser(UseCaseInterface):
    """Elimina usuario"""

    def handle(self, request: dict) -> Response:
        """
        Args:
            request (dict): Ejemplo:
                ::
                    {
                        'id': '5e404653d1c098c5177cc69d'
                    }

        Returns:
            Response:
        """
        self._request = request
        self._user = None

        valid, response = self._validate()
        if not valid:
            return response

        try:
            self._user.delete()
            response = Response(
                message=responses.OK,
                http_code=HttpCode.OK
            )
        except Exception:
            response = Response(
                message=responses.INTERNAL_SERVER_ERROR,
                http_code=HttpCode.INTERNAL_SERVER_ERROR
            )

        return response

    def _validate(self) -> (bool, SaveUserResponse):
        """Validaciones

        Returns:
            (bool, SaveUserResponse): (True, None) = Si es válido
            (False, obj) = No es válido
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, SaveUserResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )

        self._user = UserRepository.find(self._request['id'])
        if not self._user:
            return False, SaveUserResponse(
                message=responses.NOT_FOUND,
                http_code=HttpCode.NOT_FOUND
            )

        return True, None

    def __request_is_valid(self) -> (bool, dict):
        """ El request debe cumplir con la estructura,
        no debe tener keys adicionales

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            "id": {
                'type': 'string',
                'required': True,
                'mongo_id': True
            }
        }

        valid = v.validate(self._request)
        return valid, v.errors


class EditUser(UseCaseInterface):

    def handle(self, request: dict) -> SaveUserResponse:
        """
        Args:
            request (dict): Example:
                ::
                    {
                        'id': '5e404653d1c098c5177cc69d',
                        'change_password': True
                    }

        Returns:
            SaveUserResponse:
        """
        self.__request = request
        self.__user = None

        valid, response = self.__validate()
        if not valid:
            return response

        self.__edit()

        return SaveUserResponse(
            message=responses.OK,
            http_code=HttpCode.OK,
            user=self.__user
        )

    def __edit(self) -> None:
        """Edit change_password field with request data"""
        self.__user.change_password = self.__request['change_password']
        self.__user.save()

    def __validate(self) -> (bool, SaveUserResponse):
        """Validations

        Returns:
            (bool, SaveUserResponse): (True, None) = Si es válido
            (False, obj) = No es válido
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, SaveUserResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )

        self.__user = UserRepository.find(self.__request['id'])
        if not self.__user:
            return False, SaveUserResponse(
                message=responses.NOT_FOUND,
                http_code=HttpCode.NOT_FOUND
            )

        return True, None

    def __request_is_valid(self) -> (bool, dict):
        """Request must have the structure,
        it must not have additional keys

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            "id": {
                "type": "string",
                "required": True,
                "mongo_id": True
            },
            "change_password": {
                "type": "boolean",
                "required": True
            }
        }

        valid = v.validate(self.__request)
        return valid, v.errors


class ResetPassword(UseCaseInterface):

    def handle(self, request) -> ResetPasswordResponse:
        """
        Args:
            role (str)
            request (dict): Example:
                ::
                    {
                        'id': '5e404653d1c098c5177cc69d'
                    }

        Returns:
            ResetPasswordResponse:
        """
        self.__request = request
        valid, response = self.__validate()
        if not valid:
            return response

        self.__edit_password()
        return ResetPasswordResponse(
            message=responses.OK,
            http_code=HttpCode.OK,
            password=self.__password
        )

    def __edit_password(self) -> None:
        """Update user password

        Attributes:
            __password (str): password generated
        """
        self.__password = self.__random_pass()
        self.__user.password = password_hash(
            self.__password
        )
        self.__user.change_password = True
        self.__user.save()

    def __random_pass(self) -> str:
        """Password Generator

        Returns:
            (str): Password generated
        """
        length = config.RESET_PASSWORD_LEN_GENERATOR
        letters = string.ascii_lowercase
        return ''.join(random.choice(letters) for i in range(length))  # nosec

    def __validate(self) -> (bool, ResetPasswordResponse):
        """Execution of validators

        Attributes:
            __user (User):

        Returns:
            (bool, ResetPasswordResponse):
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, ResetPasswordResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )

        self.__user = UserRepository.find(self.__request['id'])
        if not self.__user:
            return False, ResetPasswordResponse(
                message=responses.NOT_FOUND,
                http_code=HttpCode.NOT_FOUND,
            )

        return True, None

    def __request_is_valid(self) -> (bool, dict):
        """The order must comply with the structure,
        must not have additional keys

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            'id': {
                'type': 'string',
                'required': True,
                'mongo_id': True
            }
        }
        valid = v.validate(self.__request)

        return valid, v.errors



from .models import (
    Role,
    User,
    WhitelistToken
)


class RoleRepository:
    """Clase para búsquedas dentro del modelo Role"""

    @staticmethod
    def find(id: str) -> Role:
        """Busca un Role por id

        Args:
            id (str)

        Returns:
            security.models.Role: o None
        """
        return Role.objects(id=id).first()

    @staticmethod
    def find_one(**kwargs) -> Role:
        """Busca un Role por parámetros dados

        Args:
            kwargs

        Returns:
            security.models.Role: o None
        """
        return Role.objects(**kwargs).first()

    @staticmethod
    def find_all(**kwargs) -> Role:
        """Busca Roles por parámetros dados

        Args:
            kwargs

        Returns:
            security.models.Role: o None
        """
        return Role.objects(**kwargs)


class UserRepository:
    """Clase para búsquedas dentro del modelo User"""

    @staticmethod
    def find(id: str) -> User:
        """Busca un User por id

        Args:
            id (str)

        Returns:
            security.models.User: o None
        """
        return User.objects(id=id).first()

    @staticmethod
    def find_one(**kwargs) -> User:
        """Busca un User

        Args:
            kwargs: Atributos del modelo

        Returns:
            security.models.User: o None
        """
        return User.objects(**kwargs).first()

    @staticmethod
    def find_all(**kwargs) -> list:
        """Busca Users

        Args:
            kwargs: Atributos del modelo

        Returns:
            list[security.models.User]
        """
        return User.objects(**kwargs)


class WhitelistTokenRepository:
    """Clase para búsquedas dentro del modelo WhitelistToken"""

    @staticmethod
    def find(id: str) -> WhitelistToken:
        """Busca un WhitelistToken por id

        Args:
            id (str)

        Returns:
            security.models.WhitelistToken: o None
        """
        return WhitelistToken.objects(id=id).first()

    @staticmethod
    def find_one(**kwargs) -> WhitelistToken:
        """Busca un WhitelistToken

        Args:
            kwargs: Atributos del modelo

        Returns:
            security.models.WhitelistToken: o None
        """
        return WhitelistToken.objects(**kwargs).first()

    @staticmethod
    def find_all(**kwargs) -> list:
        """Busca WhitelistTokens

        Args:
            kwargs: Atributos del modelo

        Returns:
            list[security.models.WhitelistToken]
        """
        return WhitelistToken.objects(**kwargs)



from .security import api_rest
from .security import models
from .security import use_cases
from .security import auth
from .security import schema
from .security.repository import UserRepository
from .security.repository import RoleRepository
from .security.repository import WhitelistTokenRepository
from .security.commands import command_auth_mongo
from .security.middlewares import MutationMiddleware
from .constants.security import Roles


__all__ = (
    'auth',
    'api_rest',
    'command_auth_mongo',
    'models',
    'schema',
    'UserRepository',
    'RoleRepository',
    'WhitelistTokenRepository',
    'use_cases',
    'MutationMiddleware',
    'Roles',
)



import abc


class LoggingInterface:
    """Contrato para el logging"""
    debug_on = False

    @abc.abstractmethod
    def debug(self, msg: str):
        pass

    @abc.abstractmethod
    def info(self, msg: str):
        pass

    @abc.abstractmethod
    def error(self, msg: str):
        pass



import graphene
from graphene.relay import Node
from graphene_mongo import MongoengineObjectType
from ..models import Role as RoleModel


__all__ = (
    'Role',
)


class RoleAttribute:
    name = graphene.String()
    permissions = graphene.JSONString()


class Role(MongoengineObjectType, RoleAttribute):
    """Role Node"""

    class Meta:
        model = RoleModel
        interfaces = (Node,)



from flask import request
from .. import auth
from ..use_cases import EditPasswordUser


def edit_password_user():
    """Edit password :obj:`User` for API Rest

    Request example
    ::
        {
            'current_password': 'pass',
            'new_password': 'new_pass',
            'password_confirmed': 'new_pass'
        }

    Returns:
        (dict, int): (data, http_code)
    """
    user = auth.current_user()
    data = request.get_json()
    data['id'] = str(user.id)

    use_case = EditPasswordUser()
    result = use_case.handle(data)

    response = {
        'message': result.message
    }
    if result.errors:
        response['errors'] = result.errors

    return response, result.http_code



from unittest import TestCase
from unittest.mock import patch
from flask_auth_service_mongo.security.models import Role
from flask_auth_service_mongo.security.use_cases import (
    NewRole,
    SaveRoleResponse
)


def resolve_patch(path: str) -> str:
    return f'flask_auth_service_mongo.security.use_cases.role.{path}'


class TestNew(TestCase):

    __request = dict(
        name='a_name',
        permissions={'a': 'b'}
    )

    def test__request_not_valid(self):
        use_case = NewRole()
        request = dict(
            key='value'
        )
        result = use_case.handle(request)

        self.assertIsInstance(result, SaveRoleResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('RoleRepository.find_one'))
    def test_role_exists(self, mock_role):

        use_case = NewRole()
        result = use_case.handle(self.__request)

        self.assertIsInstance(result, SaveRoleResponse)
        self.assertEqual(result.message, 'existing_resource')
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('Role.save'))
    @patch(resolve_patch('RoleRepository.find_one'))
    def test_role_save(self, mock_find, mock_role_save):
        # Mocks
        mock_find.return_value = None

        use_case = NewRole()
        result = use_case.handle(self.__request)

        self.assertIsInstance(result, SaveRoleResponse)
        self.assertEqual(result.message, 'created')
        self.assertEqual(result.http_code, 201)
        self.assertIsInstance(result.role, Role)



from ...constants import responses
from ...constants.enums import HttpCode
from ...utils.use_case import Response, UseCaseInterface
from ...utils.validator import Validator
from ..models import Role
from ..repository import RoleRepository


class SaveRoleResponse(Response):
    """Respuesta para los casos de uso new y update role

    Attributes:
        role (security.models.Role)
        http_code (int)
        message (str)
        errors (dict)
    """
    def __init__(self, role: Role = None, *args, **kwargs):
        self.role = role

        super(SaveRoleResponse, self).__init__(*args, **kwargs)


class NewRole(UseCaseInterface):
    """Caso de uso: Crear un Role"""

    def handle(self, request: dict) -> SaveRoleResponse:
        """
        Args:
            request (dict): Ejemplo:
                ::
                    {
                        'name': 'roleName',
                        'permissions': {'a':'b'}
                    }

        Returns:
            SaveRoleResponse:
        """
        self._request = request

        valid, response = self.__validate()
        if not valid:
            return response

        role = self._create_role()
        return SaveRoleResponse(
            message=responses.CREATED,
            http_code=HttpCode.CREATED,
            role=role
        )

    def __validate(self) -> (bool, SaveRoleResponse):
        """Execution of validators

        Returns:
            (bool, SaveRoleResponse):
        """
        valid, errors = self.__request_is_valid()
        if not valid:
            return False, SaveRoleResponse(
                message=responses.BAD_REQUEST,
                http_code=HttpCode.BAD_REQUEST,
                errors=errors
            )

        exists = RoleRepository.find_one(name=self._request['name'])
        if exists:
            return False, SaveRoleResponse(
                message=responses.EXISTING_RESOURCE,
                http_code=HttpCode.BAD_REQUEST
            )

        return True, None

    def _create_role(self) -> Role:
        """Crea un Role con los datos del self._request

        Returns:
            Role:
        """
        role = Role(**self._request)
        role.save()

        return role

    def __request_is_valid(self) -> (bool, dict):
        """ El request debe cumplir con la estructura,
        no debe tener keys adicionales

        Returns:
            (bool, dict): (is_valid, errors)
        """
        v = Validator()
        v.schema = {
            'name': {
                'type': 'string',
                'required': True
            },
            'permissions': {
                'type': 'dict'
            }
        }

        valid = v.validate(self._request)
        return valid, v.errors



from unittest import TestCase
from unittest.mock import patch, Mock
from flask_auth_service_mongo.security.utils import AccessTokens
from flask_auth_service_mongo.security.use_cases import (
    Response,
    AuthResponse,
    Login,
    Logout,
    RefreshToken
)
from flask_auth_service_mongo.security.models import User, WhitelistToken


def resolve_patch(path: str) -> str:
    return f'flask_auth_service_mongo.security.use_cases.session.{path}'


class TestLogin(TestCase):

    def test_request_not_valid(self):
        use_case = Login()
        result = use_case.handle({'key': 'value'}, 'role')
        expected = {
            'key': ['unknown field'],
            'password': ['required field'],
            'username': ['required field']
        }

        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.errors, expected)
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('UserRepository.find_one'))
    def test_user_not_found(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = None

        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, 'role')

        # asserts
        mock_user_find.assert_called_with(
            username='test_name'
        )
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'bad_credentials')
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('UserRepository.find_one'))
    def test_incorrect_role(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = Mock(name='other_role')

        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, 'role')

        # asserts
        mock_user_find.assert_called_with(
            username='test_name'
        )
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'bad_credentials')
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('password_match'))
    @patch(resolve_patch('UserRepository.find_one'))
    def test_incorrect_password(self, mock_user_find, mock_password_match):
        # Mocks
        user = Mock(password='pass')
        user.role.name = 'role'
        mock_user_find.return_value = user
        mock_password_match.return_value = False

        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, 'role')

        # asserts
        mock_user_find.assert_called_with(
            username='test_name'
        )
        mock_password_match.assert_called_with(request['password'], 'pass')
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'bad_credentials')
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('config.WHITE_LIST_TOKEN'), True)
    @patch(resolve_patch('WhitelistToken'))
    @patch(resolve_patch('token_generate'))
    @patch(resolve_patch('password_match'))
    @patch(resolve_patch('UserRepository.find_one'))
    def test_ok_with_whitelist_token(
        self,
        mock_user_find,
        mock_password_match,
        mock_token_generate,
        mock_whitelist
    ):
        # Mocks
        uuids = {
            'uuid_access': 'abc123',
            'uuid_refresh': 'abc456'
        }
        tokens = AccessTokens(
            token_type='Bearer',
            access_token='__access_token',
            refresh_token='__refresh_token',
            expires_in=10
        )
        user = Mock(
            password='hash_pass',
            change_password=False
        )
        user.role.name = 'role'
        mock_user_find.return_value = user
        mock_password_match.return_value = True
        mock_token_generate.return_value = tokens, uuids

        # Exec
        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, 'role')

        # asserts
        mock_whitelist.assert_called_with(
            uuid_access='abc123',
            uuid_refresh='abc456',
            user=user
        )
        mock_password_match.assert_called_with('test_pass', 'hash_pass')
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'ok')
        self.assertEqual(result.http_code, 200)
        self.assertIsInstance(result.tokens, AccessTokens)
        self.assertEqual(result.tokens.access_token, '__access_token')
        self.assertEqual(result.tokens.refresh_token, '__refresh_token')
        self.assertEqual(result.change_password, False)

    @patch(resolve_patch('config.WHITE_LIST_TOKEN'), False)
    @patch(resolve_patch('WhitelistToken'))
    @patch(resolve_patch('token_generate'))
    @patch(resolve_patch('password_match'))
    @patch(resolve_patch('UserRepository.find_one'))
    def test_ok_without_whitelist_token(
        self,
        mock_user_find,
        mock_password_match,
        mock_token_generate,
        mock_whitelist
    ):
        # Mocks
        uuids = {
            'uuid_access': 'abc123',
            'uuid_refresh': 'abc456'
        }
        tokens = AccessTokens(
            token_type='Bearer',
            access_token='__access_token',
            refresh_token='__refresh_token',
            expires_in=10
        )
        user = Mock(
            password='hash_pass',
            change_password=False
        )
        user.role.name = 'role'
        mock_user_find.return_value = user
        mock_password_match.return_value = True
        mock_token_generate.return_value = tokens, uuids

        # Exec
        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, 'role')

        # asserts
        mock_whitelist.assert_not_called()
        mock_password_match.assert_called_with('test_pass', 'hash_pass')
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'ok')
        self.assertEqual(result.http_code, 200)
        self.assertIsInstance(result.tokens, AccessTokens)
        self.assertEqual(result.tokens.access_token, '__access_token')
        self.assertEqual(result.tokens.refresh_token, '__refresh_token')
        self.assertEqual(result.change_password, False)

    @patch(resolve_patch('config.WHITE_LIST_TOKEN'), False)
    @patch(resolve_patch('WhitelistToken'))
    @patch(resolve_patch('token_generate'))
    @patch(resolve_patch('password_match'))
    @patch(resolve_patch('UserRepository.find_one'))
    def test_ok_list_roles(
        self,
        mock_user_find,
        mock_password_match,
        mock_token_generate,
        mock_whitelist
    ):
        # Mocks
        uuids = {
            'uuid_access': 'abc123',
            'uuid_refresh': 'abc456'
        }
        tokens = AccessTokens(
            token_type='Bearer',
            access_token='__access_token',
            refresh_token='__refresh_token',
            expires_in=10
        )
        user = Mock(
            password='hash_pass',
            change_password=False
        )
        user.role.name = 'role'
        mock_user_find.return_value = user
        mock_password_match.return_value = True
        mock_token_generate.return_value = tokens, uuids

        # Exec
        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request, ['role2', 'role'])

        # asserts
        mock_whitelist.assert_not_called()
        mock_password_match.assert_called_with('test_pass', 'hash_pass')
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'ok')
        self.assertEqual(result.http_code, 200)
        self.assertIsInstance(result.tokens, AccessTokens)
        self.assertEqual(result.tokens.access_token, '__access_token')
        self.assertEqual(result.tokens.refresh_token, '__refresh_token')
        self.assertEqual(result.change_password, False)

    @patch(resolve_patch('config.WHITE_LIST_TOKEN'), False)
    @patch(resolve_patch('WhitelistToken'))
    @patch(resolve_patch('token_generate'))
    @patch(resolve_patch('password_match'))
    @patch(resolve_patch('UserRepository.find_one'))
    def test_ok_without_roles(
        self,
        mock_user_find,
        mock_password_match,
        mock_token_generate,
        mock_whitelist
    ):
        # Mocks
        uuids = {
            'uuid_access': 'abc123',
            'uuid_refresh': 'abc456'
        }
        tokens = AccessTokens(
            token_type='Bearer',
            access_token='__access_token',
            refresh_token='__refresh_token',
            expires_in=10
        )
        user = Mock(
            password='hash_pass',
            change_password=False
        )
        user.role.name = 'role'
        mock_user_find.return_value = user
        mock_password_match.return_value = True
        mock_token_generate.return_value = tokens, uuids

        # Exec
        use_case = Login()
        request = dict(
            username='test_name',
            password='test_pass'
        )
        result = use_case.handle(request)

        # asserts
        mock_whitelist.assert_not_called()
        mock_password_match.assert_called_with('test_pass', 'hash_pass')
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'ok')
        self.assertEqual(result.http_code, 200)
        self.assertIsInstance(result.tokens, AccessTokens)
        self.assertEqual(result.tokens.access_token, '__access_token')
        self.assertEqual(result.tokens.refresh_token, '__refresh_token')
        self.assertEqual(result.change_password, False)


class TestLogout(TestCase):

    @patch(resolve_patch('token_decode'))
    @patch(resolve_patch('WhitelistTokenRepository.find_one'))
    def test_ok(self, mock_find, mock_decode):
        # Mocks
        mock_token = Mock()
        mock_token.delete.return_value = True
        mock_find.return_value = mock_token
        mock_result = Mock(
            user_id='user1',
            uuid='u123'
        )
        mock_decode.return_value = mock_result

        use_case = Logout()
        result = use_case.handle('a_token')

        # Asserts
        self.assertIsInstance(result, Response)
        self.assertEqual(result.http_code, 200)
        self.assertEqual(result.message, 'ok')
        mock_find.assert_called_with(uuid_access='u123')
        mock_token.delete.assert_called_with()


class TestRefreshToken(TestCase):

    def test_request_not_valid(self):
        # Exec
        use_case = RefreshToken()
        result = use_case.handle({'key': 'value'})

        # Asserts
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)
        self.assertEqual(result.errors, {
            'key': ['unknown field'],
            'refresh_token': ['required field']
        })

    def test_payload_error(self):
        # Exec
        use_case = RefreshToken()
        result = use_case.handle({
            'refresh_token': 'a_refresh_token'
        })

        # Asserts
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'invalid_token')
        self.assertEqual(result.http_code, 401)

    @patch(resolve_patch('current_user'))
    @patch(resolve_patch('refresh_token_decode'))
    @patch(resolve_patch('WhitelistTokenRepository.find_one'))
    def test_refresh_token_not_saved(
        self,
        mock_find,
        mock_decode,
        mock_current_user
    ):
        # Mocks
        mock_decode.return_value = Mock(
            uuid='u123',
            error=None
        )
        mock_find.return_value = None
        mock_current_user.return_value = User(
            id='5e6a9440a3d71505cc416ed6'
        )

        # Exec
        use_case = RefreshToken()
        result = use_case.handle({
            'refresh_token': 'a_refresh_token'
        })

        # Asserts
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'unauthorized')
        self.assertEqual(result.http_code, 401)
        mock_find.assert_called_with(uuid_refresh='u123')
        mock_decode.assert_called_with('a_refresh_token')

    @patch(resolve_patch('current_user'))
    @patch(resolve_patch('refresh_token_decode'))
    @patch(resolve_patch('WhitelistTokenRepository.find_one'))
    def test_current_user_not_is_user_refresh_token(
        self,
        mock_find,
        mock_decode,
        mock_current_user
    ):
        # Mocks
        mock_decode.return_value = Mock(
            uuid='u123',
            error=None
        )
        mock_find.return_value = WhitelistToken(
            user=User(
                id='5e6a93b924beeae45a431af3'
            )
        )
        mock_current_user.return_value = User(
            id='5e6a9440a3d71505cc416ed6'
        )

        # Exec
        use_case = RefreshToken()
        result = use_case.handle({
            'refresh_token': 'a_refresh_token'
        })

        # Asserts
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'unauthorized')
        self.assertEqual(result.http_code, 401)
        mock_current_user.assert_called_with()

    @patch(resolve_patch('WhitelistToken'))
    @patch(resolve_patch('token_generate'))
    @patch(resolve_patch('current_user'))
    @patch(resolve_patch('refresh_token_decode'))
    @patch(resolve_patch('WhitelistTokenRepository.find_one'))
    def test_ok(
        self,
        mock_find,
        mock_decode,
        mock_current_user,
        mock_token_generate,
        mock_whitelist
    ):
        # Mocks
        mock_decode.return_value = Mock(
            uuid='u123',
            error=None
        )
        user = User(
            id='5e6a93b924beeae45a431af3'
        )
        mock_find.return_value = WhitelistToken(
            user=user
        )
        mock_current_user.return_value = user
        uuids = {
            'uuid_access': 'abc123',
            'uuid_refresh': 'abc456'
        }
        tokens = AccessTokens(
            token_type='Bearer',
            access_token='__access_token',
            refresh_token='__refresh_token',
            expires_in=10
        )
        mock_token_generate.return_value = tokens, uuids

        # Exec
        use_case = RefreshToken()
        result = use_case.handle({
            'refresh_token': 'a_refresh_token'
        })

        # Asserts
        self.assertIsInstance(result, AuthResponse)
        self.assertEqual(result.message, 'ok')
        self.assertEqual(result.http_code, 200)
        self.assertIsInstance(result.tokens, AccessTokens)
        self.assertEqual(result.tokens.access_token, '__access_token')
        self.assertEqual(result.tokens.refresh_token, '__refresh_token')
        mock_whitelist.assert_called_with(
            uuid_access='abc123',
            uuid_refresh='abc456',
            user=user
        )



from enum import Enum, unique


@unique
class HttpCode(Enum):
    # 2xx Success
    OK = 200
    CREATED = 201
    ACCEPTED = 202

    # 4xx Client Error
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    NOT_ACCEPTABLE = 406
    NOT_FOUND = 404

    # 5xx Server Error
    INTERNAL_SERVER_ERROR = 500



from flask import g
from ..models import User


def current_user() -> User:
    """Retorna el usuario autenticado actual

    Returns:
        security.models.User: `None` or :class:`security.models.User` object
    """
    return g.get('user')


def set_current_user(user: User) -> None:
    """Define el usuario actual,
    para poder usar security.auth.current_user()

    Args:
        user (security.models.User):
    """
    g.user = user



from unittest import TestCase
from unittest.mock import patch
from flask_auth_service_mongo.security.repository import (
    RoleRepository,
    UserRepository,
    WhitelistTokenRepository
)


def resolve_patch(path: str) -> str:
    return 'flask_auth_service_mongo.security.repository.{}'.format(path)


class TestRoleRepository(TestCase):

    @patch(resolve_patch('Role.objects'))
    def test_find(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = RoleRepository.find(id='abc123')

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(id='abc123')

    @patch(resolve_patch('Role.objects'))
    def test_find_one(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = RoleRepository.find_one(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(key='abc123', other=123)

    @patch(resolve_patch('Role.objects'))
    def test_find_all(self, mock_obj):
        # Mocks
        mock_obj.return_value = ['test']

        result = RoleRepository.find_all(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, ['test'])
        mock_obj.assert_called_with(key='abc123', other=123)


class TestUserRepository(TestCase):

    @patch(resolve_patch('User.objects'))
    def test_find(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = UserRepository.find(id='abc123')

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(id='abc123')

    @patch(resolve_patch('User.objects'))
    def test_find_one(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = UserRepository.find_one(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(key='abc123', other=123)

    @patch(resolve_patch('User.objects'))
    def test_find_all(self, mock_obj):
        # Mocks
        mock_obj.return_value = ['test']

        result = UserRepository.find_all(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, ['test'])
        mock_obj.assert_called_with(key='abc123', other=123)


class TestWhitelistTokenRepository(TestCase):

    @patch(resolve_patch('WhitelistToken.objects'))
    def test_find(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = WhitelistTokenRepository.find(id='abc123')

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(id='abc123')

    @patch(resolve_patch('WhitelistToken.objects'))
    def test_find_one(self, mock_obj):
        # Mocks
        mock_obj.return_value.first.return_value = 'test'

        result = WhitelistTokenRepository.find_one(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, 'test')
        mock_obj.assert_called_with(key='abc123', other=123)

    @patch(resolve_patch('WhitelistToken.objects'))
    def test_find_all(self, mock_obj):
        # Mocks
        mock_obj.return_value = ['test']

        result = WhitelistTokenRepository.find_all(key='abc123', other=123)

        # Asserts
        self.assertEqual(result, ['test'])
        mock_obj.assert_called_with(key='abc123', other=123)



# HTTP messages
BAD_REQUEST = 'bad_request'
OK = 'ok'
UNAUTHORIZED = 'unauthorized'
CREATED = 'created'
NOT_ACCEPTABLE = 'not_acceptable'
NOT_FOUND = 'not_found'
INTERNAL_SERVER_ERROR = 'internal_server_error'

# Actions
SAVED = 'saved'

# Messages
BAD_CREDENTIALS = 'bad_credentials'
SIGNATURE_EXPIRED = 'signature_expired'
INVALID_TOKEN = 'invalid_token'
ERROR = 'error'
EXISTING_RESOURCE = 'existing_resource'
ID_NOT_VALID = 'id_not_valid'



from flask import request
from ..use_cases import Login, Logout


def login(role=None):
    """login for API Rest

    Args:
        role (str|List[str]): Nullable

    Returns:
        (dict, int): (data, http_code)
    """
    use_case = Login()
    result = use_case.handle(
        request=request.get_json(),
        role=role
    )

    response = dict(
        message=result.message
    )
    if result.tokens:
        response['data'] = dict(
            access_token=result.tokens.access_token,
            token_type=result.tokens.token_type,
            refresh_token=result.tokens.refresh_token,
            expires_in=result.tokens.expires_in,
            change_password=result.change_password
        )
    return response, result.http_code


def logout():
    """logout for API Rest

    Returns:
        (dict, int): (data, http_code)
    """
    authorization = request.headers.get('Authorization')
    _, token = authorization.split(' ')
    use_case = Logout()
    result = use_case.handle(token=token)

    response = dict(message=result.message)
    return response, result.http_code



from unittest import TestCase
from unittest.mock import patch, Mock
from flask_auth_service_mongo.security.use_cases import (
    CreateUser,
    SaveUserResponse,
    EditUser,
    EditPasswordUser,
    DeleteUser,
    ResetPassword,
    Response,
    ResetPasswordResponse
)
from flask_auth_service_mongo.security.models import User, Role


def resolve_patch(path: str) -> str:
    return 'flask_auth_service_mongo.security.use_cases.user.{}'.format(path)


class TestCreateUser(TestCase):

    def test_request_not_valid(self):
        use_case = CreateUser()
        result = use_case.handle({'key': 'value'})

        expected = {
            'key': ['unknown field'],
            'password': ['required field'],
            'password_confirmed': ['required field'],
            'role': ['required field'],
            'username': ['required field']
        }

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.errors, expected)
        self.assertEqual(result.http_code, 400)

    def test_username_not_valid(self):
        use_case = CreateUser()
        request = {
            'username': '12',
            'password': '1234',
            'password_confirmed': '1234',
            'role': 'any'
        }
        result = use_case.handle(request)

        expected = {'username': [
            'min length is 3']
        }

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.errors, expected)
        self.assertEqual(result.http_code, 400)

    def test_passwords_not_match(self):
        request = dict(
            username='str',
            role='str',
            password='pass',
            password_confirmed='1234'
        )
        use_case = CreateUser()
        result = use_case.handle(request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "Passwords don't match")
        self.assertEqual(result.http_code, 406)

    @patch(resolve_patch('RoleRepository.find_one'))
    def test_rol_not_found(self, mock_role_find):
        # Mocks
        mock_role_find.return_value = None

        request = dict(
            username='str',
            role='notExists',
            password='pass',
            password_confirmed='pass'
        )
        use_case = CreateUser()
        result = use_case.handle(request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "Role does not exist")
        self.assertEqual(result.http_code, 406)
        mock_role_find.assert_called_with(name='notExists')

    @patch(resolve_patch('UserRepository.find_one'))
    @patch(resolve_patch('RoleRepository.find_one'))
    def test_user_repeated(self, mock_role_find, mock_user_find):
        request = dict(
            username='exisTs',
            role='rol',
            password='pass',
            password_confirmed='pass'
        )
        use_case = CreateUser()
        result = use_case.handle(request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "Username already exists")
        self.assertEqual(result.http_code, 406)
        mock_user_find.assert_called_with(username='exists')

    @patch(resolve_patch('User.save'))
    @patch(resolve_patch('UserRepository.find_one'))
    @patch(resolve_patch('RoleRepository.find_one'))
    def test_ok(self, mock_role_find, mock_user_find, mock_user_save):
        # Mocks
        mock_user_find.return_value = None
        mock_role_find.return_value = Role()

        request = dict(
            username='str',
            role='a role',
            password='pass',
            password_confirmed='pass'
        )
        use_case = CreateUser()
        result = use_case.handle(request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "created")
        self.assertEqual(result.http_code, 201)
        self.assertIsInstance(result.user, User)
        mock_user_save.assert_called_with()

    @patch(resolve_patch('User.save'), *{
        'return_value.objects.side_effect': Exception()
    })
    @patch(resolve_patch('UserRepository.find_one'))
    @patch(resolve_patch('RoleRepository.find_one'))
    def test_except(self, mock_role_find, mock_user_find):
        # Mocks
        mock_user_find.return_value = None
        mock_role_find.return_value = Role()

        request = dict(
            username='str',
            role='aRole',
            password='pass',
            password_confirmed='pass'
        )
        use_case = CreateUser()
        result = use_case.handle(request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "internal_server_error")
        self.assertEqual(result.http_code, 500)
        self.assertIsNone(result.user)


class TestEditPassword(TestCase):

    ID = '5e615cd4ec0af8597065055d'
    PASS = '$2b$12$umfFAKhJSKiB0jjP/a.yJeCdurFQNyWik3Cdg.ccLf4iJYD09zQ7e'

    def test_request_not_valid(self):
        use_case = EditPasswordUser()
        result = use_case.handle({'key': 'value'})

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)
        self.assertEqual(result.errors, {
            'key': ['unknown field'],
            'id': ['required field'],
            'current_password': ['required field'],
            'new_password': ['required field'],
            'password_confirmed': ['required field']
        })

    def test_passwords_not_valid(self):
        use_case = EditPasswordUser()
        # pass con espacios
        request = dict(
            id=self.ID,
            current_password='currentP',
            new_password=' d',
            password_confirmed='123'
        )
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)
        self.assertEqual(result.errors, {
            'new_password': [
                'Must not contain spaces',
                'min length is 3'
            ]
        })

    def test_passwords_not_match(self):
        request = dict(
            id=self.ID,
            current_password='currentP',
            new_password='pass',
            password_confirmed='1234'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "Passwords don't match")
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('UserRepository.find'))
    def test_not_found(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = None

        request = dict(
            id=self.ID,
            current_password='currentP',
            new_password='pass',
            password_confirmed='pass'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'not_found')
        self.assertEqual(result.http_code, 404)
        mock_user_find.assert_called_with(self.ID)

    @patch(resolve_patch('UserRepository.find'))
    def test_current_password_not_match(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = User(
            password=self.PASS
        )

        request = dict(
            id=self.ID,
            current_password='otherPass',
            new_password='newPass',
            password_confirmed='newPass'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "Current password don't match")
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('UserRepository.find'))
    def test_new_password_is_current(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = User(
            password=self.PASS
        )

        request = dict(
            id=self.ID,
            current_password='pass',
            new_password='pass',
            password_confirmed='pass'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(
            result.message,
            'The new password is equal as the current password'
        )
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('User.save'))
    @patch(resolve_patch('UserRepository.find'))
    def test_ok(self, mock_user_find, mock_user_save):
        # Mocks
        user = User(
            password=self.PASS,
            change_password=True
        )
        mock_user_find.return_value = user

        request = dict(
            id=self.ID,
            current_password='pass',
            new_password='newPass',
            password_confirmed='newPass'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "ok")
        self.assertEqual(result.http_code, 200)
        self.assertEqual(result.user, user)
        self.assertNotEqual(user.password, self.PASS)
        self.assertFalse(user.change_password)
        mock_user_save.assert_called_with()

    @patch(resolve_patch('User.save'), *{
        'return_value.objects.side_effect': Exception()
    })
    @patch(resolve_patch('UserRepository.find'))
    def test_except(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = User(
            password=self.PASS
        )

        request = dict(
            id=self.ID,
            current_password='pass',
            new_password='newPass',
            password_confirmed='newPass'
        )
        use_case = EditPasswordUser()
        result = use_case.handle(request=request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "internal_server_error")
        self.assertEqual(result.http_code, 500)
        self.assertIsNone(result.user)


class TestDeleteUser(TestCase):

    ID = '5e404653d1c098c5177cc69d'

    def test_request_not_valid(self):
        use_case = DeleteUser()
        result = use_case.handle({'key': 'value'})

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)
        self.assertEqual(result.errors, {
            'key': ['unknown field'],
            'id': ['required field']
        })

    @patch(resolve_patch('UserRepository.find'))
    def test_not_found(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = None

        # Exec
        use_case = DeleteUser()
        result = use_case.handle({'id': self.ID})

        # Asserts
        self.assertIsInstance(result, Response)
        self.assertEqual(result.message, 'not_found')
        self.assertEqual(result.http_code, 404)
        mock_user_find.assert_called_with(self.ID)

    @patch(resolve_patch('User.delete'))
    @patch(resolve_patch('UserRepository.find'))
    def test_ok(self, mock_user_find, mock_user_delete):
        # Mocks
        mock_user_find.return_value = User()

        # Exec
        use_case = DeleteUser()
        result = use_case.handle({'id': self.ID})

        # asserts
        self.assertIsInstance(result, Response)
        self.assertEqual(result.message, "ok")
        self.assertEqual(result.http_code, 200)
        mock_user_delete.assert_called_with()

    @patch(resolve_patch('User.delete'), *{
        'return_value.objects.side_effect': Exception()
    })
    @patch(resolve_patch('UserRepository.find'))
    def test_except(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = User()

        # Exec
        use_case = DeleteUser()
        result = use_case.handle({'id': self.ID})

        # asserts
        self.assertIsInstance(result, Response)
        self.assertEqual(result.message, "internal_server_error")
        self.assertEqual(result.http_code, 500)


class TestEditUser(TestCase):

    ID = '5e615cd4ec0af8597065055d'

    def test_request_not_valid(self):
        use_case = EditUser()
        result = use_case.handle({'key': 'value'})

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.http_code, 400)
        self.assertEqual(result.errors, {
            'key': ['unknown field'],
            'id': ['required field'],
            'change_password': ['required field']
        })

    @patch(resolve_patch('UserRepository.find'))
    def test_not_found(self, mock_user_find):
        # Mocks
        mock_user_find.return_value = None

        request = dict(
            id=self.ID,
            change_password=True,
        )
        use_case = EditUser()
        result = use_case.handle(request=request)

        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, 'not_found')
        self.assertEqual(result.http_code, 404)
        mock_user_find.assert_called_with(self.ID)

    @patch(resolve_patch('User.save'))
    @patch(resolve_patch('UserRepository.find'))
    def test_ok(self, mock_user_find, mock_user_save):
        # Mocks
        user = User(
            username='user',
            change_password=True
        )
        mock_user_find.return_value = user

        request = dict(
            id=self.ID,
            change_password=False,
        )
        use_case = EditUser()
        result = use_case.handle(request=request)

        # asserts
        self.assertIsInstance(result, SaveUserResponse)
        self.assertEqual(result.message, "ok")
        self.assertEqual(result.http_code, 200)
        self.assertEqual(result.user, user)
        self.assertFalse(user.change_password)
        mock_user_save.assert_called_with()


class TestResetPassword(TestCase):

    def test_bad_request(self):
        use_case = ResetPassword()
        result = use_case.handle({
            'key': 'value'
        })

        self.assertIsInstance(result, ResetPasswordResponse)
        self.assertEqual(result.message, 'bad_request')
        self.assertEqual(result.errors, {
            'id': ['required field'],
            'key': ['unknown field']
        })
        self.assertEqual(result.http_code, 400)

    @patch(resolve_patch('UserRepository.find'))
    def test_not_found(self, mock_repo):
        mock_repo.return_value = None

        use_case = ResetPassword()
        request = {'id': '5e404653d1c098c5177cc69d'}
        result = use_case.handle(request)

        self.assertIsInstance(result, ResetPasswordResponse)
        self.assertEqual(result.message, 'not_found')
        self.assertIsNone(result.errors)
        self.assertEqual(result.http_code, 404)

    @patch(resolve_patch('UserRepository.find'))
    def test_ok(self, mock_repo):
        mock_result = Mock()
        mock_result.return_value = User(
            password='something', change_password=False)
        mock_result.save.return_value = None
        mock_repo.return_value = mock_result

        use_case = ResetPassword()
        request = {'id': '5e404653d1c098c5177cc69d'}
        result = use_case.handle(request)

        self.assertIsInstance(result, ResetPasswordResponse)
        self.assertEqual(result.message, 'ok')
        self.assertIsNone(result.errors)
        self.assertEqual(result.http_code, 200)
        mock_result.save.assert_called_with()
        self.assertNotEqual(mock_result.password, 'something')
        self.assertEqual(mock_result.change_password, True)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="flask_auth_service_mongo",
    version="0.1.7b1",
    author="Terminus",
    author_email="mateo.chaparro@zinobe.com",
    description="Flask JWT authentication package with mongo.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/terminus-zinobe/flask-auth-service-mongo",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'bcrypt>=3,<4',
        'Cerberus>=1.3,<2',
        'flask>=1,<2',
        'PyJWT>=1.7,<2',
        'mongoengine>=0.18,<0.20',
        'radon>=4,<5',
        'graphene>=2.1,<3',
        'graphene-mongo>=0.2,<0.3',
    ]
)



from unittest import TestCase
from unittest.mock import patch, Mock
from flask import Flask
from flask_auth_service_mongo.security.models import User
from flask_auth_service_mongo.security.utils import Payload
from flask_auth_service_mongo.security.auth import (
    required,
    current_user,
    set_current_user
)


def resolve_patch(path: str) -> str:
    return 'flask_auth_service_mongo.security.auth.{}'.format(path)


class TestCurrentUser(TestCase):
    def test_user(self):
        # Crea el contexto del app flask
        app = Flask(__name__)
        app.config['SECRET_KEY'] = 'not-secret'

        with app.app_context():
            user = current_user()
            self.assertIsNone(user)

            param = User()
            set_current_user(param)
            user = current_user()
            self.assertEqual(user, param)


class TestRequired(TestCase):

    @required('test')
    def with_required(self):
        """funcion que usa el decorador a probar"""
        return {'message': 'ok'}, 200

    @required(['test', 'other'])
    def with_required_roles(self):
        """funcion que usa el decorador a probar"""
        return {'message': 'ok'}, 200

    @required(['test', 'other'], True)
    def with_require_password_change(self):
        """funcion que usa el decorador a probar"""
        return {'message': 'ok'}, 200

    @patch(resolve_patch('decorators.request'))
    def test_header_without_authorization(self, mock_request):
        """# header sin "Authorization"""
        # mock request.header
        mock_request.headers.get = Mock(return_value=None)

        result = self.with_required()

        # Prueba header que se esta usando para obtener el token
        args, kwargs = mock_request.headers.get.call_args
        self.assertEqual(args, ('Authorization',))
        self.assertEqual(kwargs, {})

        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.request'))
    def test_error_token_type(self, mock_request):
        """# Tipo de "Authorization" no valido"""
        # mock request.header
        mock_request.headers.get = Mock(return_value='BearerXD abc')

        result = self.with_required()

        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_error_token(self, mock_request, mock_token_decode):
        """# Error de token"""
        payload = Payload(error='error')
        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload

        result = self.with_required()

        # Prueba header que se esta usando para obtener el token
        args, kwargs = mock_token_decode.call_args
        self.assertEqual(args, ('abc',))
        self.assertEqual(kwargs, {})

        self.assertEqual(result, ({'message': 'error'}, 401))

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_token_in_whitelist_nod_found(
        self,
        mock_request,
        mock_token_decode,
        mock_whitelist_find
    ):
        """# Token no esta en WhitelistToken"""
        payload = Payload(user_id='a1', _uuid='U123')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        mock_whitelist_find.return_value = None

        result = self.with_required()

        mock_whitelist_find.assert_called_with(uuid_access='U123')
        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_user_not_found(
        self,
        mock_request,
        mock_token_decode,
        mock_whitelist_find,
        mock_user_find
    ):
        """# Usuario no encontrado"""
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        mock_user_find.return_value = None
        mock_whitelist_find.return_value = Mock()

        result = self.with_required()

        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_error_role(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_whitelist_find
    ):
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock()
        user.role.name = 'other_role'
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        result = self.with_required()

        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_error_roles(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_whitelist_find
    ):
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock()
        user.role.name = 'other_role'
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        result = self.with_required_roles()

        self.assertEqual(result, ({'message': 'unauthorized'}, 401))

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_error_password_change_required(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_whitelist_find
    ):
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock(change_password=True)
        user.role.name = 'test'
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        result = self.with_require_password_change()

        self.assertEqual(
            result,
            ({'message': 'password_change_required'}, 401)
        )

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.set_current_user'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_ok(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_set_current_user,
        mock_whitelist_find
    ):
        """# Usuario no encontrado"""
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock()
        user.role = Mock()
        user.role.name = 'test'
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        result = self.with_required()

        # Prueba que se ejecute la funcion decorada
        self.assertEqual(result, ({'message': 'ok'}, 200))

        # Prueba que se haga el set_current_user
        args, kwargs = mock_set_current_user.call_args
        self.assertEqual(args, (user,))
        self.assertEqual(kwargs, {})

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.set_current_user'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_ok_multi_roles(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_set_current_user,
        mock_whitelist_find
    ):
        """# Usuario no encontrado"""
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock()
        user.role = Mock()
        user.role.name = 'test'
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        result = self.with_required_roles()

        # Prueba que se ejecute la funcion decorada
        self.assertEqual(result, ({'message': 'ok'}, 200))

        # Prueba que se haga el set_current_user
        args, kwargs = mock_set_current_user.call_args
        self.assertEqual(args, (user,))
        self.assertEqual(kwargs, {})

    @patch(resolve_patch('decorators.WhitelistTokenRepository.find_one'))
    @patch(resolve_patch('decorators.set_current_user'))
    @patch(resolve_patch('decorators.UserRepository.find'))
    @patch(resolve_patch('decorators.token_decode'))
    @patch(resolve_patch('decorators.request'))
    def test_ok_without_role(
        self,
        mock_request,
        mock_token_decode,
        mock_user_find,
        mock_set_current_user,
        mock_whitelist_find
    ):
        """# Usuario no encontrado"""
        payload = Payload(user_id='a1')

        # mocks
        mock_request.headers.get = Mock(return_value='Bearer abc')
        mock_token_decode.return_value = payload
        user = Mock()
        mock_user_find.return_value = user
        mock_whitelist_find.return_value = Mock()

        @required()  # Sin role
        def with_required():
            """funcion que usa el decorador a probar"""
            return {'message': 'created'}, 201

        result = with_required()

        # Prueba que se ejecute la funcion decorada
        self.assertEqual(result, ({'message': 'created'}, 201))

        # Prueba que se haga el set_current_user
        args, kwargs = mock_set_current_user.call_args
        self.assertEqual(args, (user,))
        self.assertEqual(kwargs, {})



from ..constants.security import Roles
from .auth import current_user


class MutationMiddleware:
    """
    Args:
        access_control (List[Dict]): Example:
            ::
                [{
                    'mutation': 'login',
                    'roles': ['is_anonymously']
                }, {
                    'mutation': 'logout',
                    'roles': ['is_authenticated']
                }, {
                    'mutation': 'create_obj',
                    'roles': ['custom_role']
                }]
    """

    def __init__(self, access_control: list):
        self.access_control = access_control

    def resolve(self, next, root, info, **args):
        if root is not None or info.operation.operation != 'mutation':
            # Not a mutation, everything is allowed
            return next(root, info, **args)

        access_control = self.__get_access_control(info.field_name)
        if access_control is None:
            # No access rules were defined for this mutation
            return next(root, info, **args)

        user = current_user()
        if not user:
            if Roles.IS_AUTHENTICATED_ANONYMOUSLY in access_control['roles']:
                # user anonymously
                return next(root, info, **args)
            else:
                # Unauthorized
                return None

        if Roles.IS_AUTHENTICATED_FULLY in access_control['roles']:
            # Mutation usable by any authenticated user
            return next(root, info, **args)

        if user.role.name not in access_control['roles']:
            # Role not valid
            return None

        # Role ok
        return next(root, info, **args)

    def __get_access_control(self, field_name: str) -> dict:
        """Returns the first access_control when
        field_name == access_control.mutation

        Args:
            field_name (str):

        Returns:
            dict: Or None
        """
        access_control = list(filter(
            lambda d: d['mutation'] == field_name,
            self.access_control
        ))
        if access_control:
            return access_control[0]
        return None



from graphql import GraphQLError
from graphql_relay.node.node import from_global_id
from ..constants import responses


def input_to_dictionary(input):
    """Method to convert Graphene inputs to a dictionary"""
    dictionary = {}
    for key in input:
        # Convert GraphQL global id to database id
        if key == 'id':
            try:
                global_id = from_global_id(input[key])
            except Exception:
                raise GraphQLError(responses.ID_NOT_VALID)
            if isinstance(global_id, tuple) and len(global_id) == 2:
                input[key] = global_id[1]
            else:
                raise GraphQLError(responses.ID_NOT_VALID)
        if input[key] is not None:
            dictionary[key] = input[key]
    return dictionary
